@@ -6,7 +6,7 @@
 #include <telepathy-glib/proxy.h>
 
 #include "api/_gen/enums.h"
-#include "api/_gen/cli-misc.h"
+#include "api/_gen/cli-videocontrol.h"
 #include "api/_gen/svc-misc.h"
 
 G_BEGIN_DECLS
