@@ -20,11 +20,15 @@
 
 #include "config.h"
 
-#define USE_REALTIME
-#ifdef USE_REALTIME
+#define _GNU_SOURCE
 #include <sched.h>
 #include <sys/mman.h>
-#endif /* USE_REALTIME */
+#include <sys/resource.h>
+#include <sys/time.h>
+
+#ifndef RLIMIT_RTTIME
+# define RLIMIT_RTTIME 15
+#endif
 
 #include <signal.h>
 #include <stdlib.h>
@@ -38,14 +42,16 @@
 #include <glib.h>
 #include <glib/gstdio.h>
 
+#include <gtk/gtk.h>
+
 #include <dbus/dbus-glib.h>
 #include <gst/gst.h>
 
 #include <telepathy-glib/debug.h>
 #include <telepathy-glib/errors.h>
 
+#include "api/api.h"
 #include "tp-stream-engine.h"
-#include "channel.h"
 
 GSource *timeout = NULL;
 GMainLoop *mainloop = NULL;
@@ -64,52 +70,6 @@ gboolean forced_exit_in_progress = FALSE;
 #define WATCHDOG_BARK 5
 #define WATCHDOG_BITE 30
 
-#ifdef USE_REALTIME
-#define DEF_PRIORITY_POLICY SCHED_RR
-#define PRIORITY_DELTA 1
-
-static void
-set_realtime (const char *argv0, int policy) {
-  int orig_uid, orig_euid;
-  int prio_policy;
-  int prio_delta = PRIORITY_DELTA;
-  struct sched_param schedp;
-
-  /* get original uid */
-  orig_uid = getuid();
-  orig_euid = geteuid();
-  /* set uid to root */
-  if (setreuid(orig_uid, 0) == -1) {
-    perror("setreuid()");
-    g_warning("unable to setreuid(,0), maybe you should: \n");
-    g_warning("\tchown root %s ; chmod u+s %s\n", argv0, argv0);
-  }
-  /* set scheduling parameters, scheduler either SCHED_RR or SCHED_FIFO */
-  switch (policy) {
-    case 1:
-      prio_policy = SCHED_RR;
-      break;
-    case 2:
-      prio_policy = SCHED_FIFO;
-      break;
-    default:
-      prio_policy = DEF_PRIORITY_POLICY;
-  }
-  memset(&schedp, 0x00, sizeof(schedp));
-  schedp.sched_priority = sched_get_priority_min(prio_policy) + prio_delta;
-  /* 0 pid equals to getpid() ie. current process */
-  if (sched_setscheduler(0, prio_policy, &schedp) == -1) {
-    perror("sched_setscheduler()");
-  }
-  /* nail everything to RAM, needed for realtime on systems with swap,
-   * also avoids extra calls to vm subsystem */
-  /*if (mlockall(MCL_CURRENT|MCL_FUTURE) == -1) {
-    perror("mlockall()");
-  }*/
-  /* restore original uid */
-  setreuid(orig_uid, orig_euid);
-}
-#endif /* USE_REALTIME */
 
 static gboolean
 kill_stream_engine (gpointer data G_GNUC_UNUSED)
@@ -157,51 +117,27 @@ dsp_crashed (gpointer dummy)
 {
   if (stream_engine)
   {
-    tp_stream_engine_error (stream_engine, 0, "DSP Crash");
+    tp_stream_engine_error (stream_engine, TP_MEDIA_STREAM_ERROR_MEDIA_ERROR,
+        "DSP Crash");
     g_object_unref (stream_engine);
     g_main_loop_quit (mainloop);
   }
 }
 #endif
 
-#ifdef GST_DEBUG_BIN_TO_DOT_FILE_WITH_TS
-
-static void
-stream_engine_channel_dump_stream (TpStreamEngineChannel *chan,
-    guint stream_id, TpStreamEngineStream *stream, gpointer _user_data)
-{
-  GstElement *pipeline = NULL;
-  gchar *name;
-
-  pipeline = farsight_stream_get_pipeline (stream->fs_stream);
-
-  if (pipeline == NULL)
-    return;
-
-  name = g_strdup_printf("%s-channel-%p-%s",
-    TP_STREAM_ENGINE_IS_VIDEO_STREAM(stream) ? "video" : "audio", chan,
-    GST_OBJECT_NAME(pipeline));
-
-  GST_DEBUG_BIN_TO_DOT_FILE_WITH_TS (GST_BIN (pipeline),
-    GST_DEBUG_GRAPH_SHOW_ALL, name);
-
-  g_free (name);
-}
-
 static gboolean
-dump_dot_files(gpointer data)
+dump_dot_file (gpointer data G_GNUC_UNUSED)
 {
   TpStreamEngine *engine = tp_stream_engine_get ();
-  GPtrArray *channels;
-  guint i;
+  GstElement *pipeline = NULL;
 
-  channels = tp_stream_engine_get_channels (engine);
-  for (i = 0; i < channels->len; i++)
+  g_object_get (engine, "pipeline", &pipeline, NULL);
+
+  if (pipeline)
     {
-      tp_stream_engine_channel_foreach_stream (
-        TP_STREAM_ENGINE_CHANNEL(g_ptr_array_index (channels, i)),
-        stream_engine_channel_dump_stream,
-        NULL);
+      GST_DEBUG_BIN_TO_DOT_FILE_WITH_TS (GST_BIN (pipeline),
+          GST_DEBUG_GRAPH_SHOW_ALL, "stream-engine-pipeline");
+      gst_object_unref (pipeline);
     }
 
   return FALSE;
@@ -210,27 +146,7 @@ dump_dot_files(gpointer data)
 static void
 got_sigusr1 (int i G_GNUC_UNUSED)
 {
-  g_idle_add (dump_dot_files, NULL);
-}
-
-#endif
-
-static void
-got_sigbus (int i G_GNUC_UNUSED)
-{
-  const char *msg = "stream engine: DSP crashed\n";
-
-  write (STDERR_FILENO, msg, strlen (msg));
-
-#if 0
-  if (!forced_exit_in_progress)
-    {
-      forced_exit_in_progress =TRUE;
-      g_idle_add ((GSourceFunc) dsp_crashed, NULL);
-    }
-#endif
-
-  _exit (1);
+  g_idle_add (dump_dot_file, NULL);
 }
 
 /* every time the watchdog barks, schedule a bite */
@@ -251,8 +167,52 @@ watchdog_bite (int sig G_GNUC_UNUSED)
 
 int main(int argc, char **argv)
 {
+  GOptionContext *optcontext;
+  GError *error = NULL;
 
-  gst_init (&argc, &argv);
+  uid_t uid;
+  struct rlimit rl;
+
+  rl.rlim_max = 15;
+  rl.rlim_cur = 15;
+  if (setrlimit(RLIMIT_RTPRIO, &rl) < 0)
+    g_warning("setrlimit rtprio: %s", strerror (errno));
+
+  rl.rlim_max = 15;
+  rl.rlim_cur = 15;
+  if (setrlimit(RLIMIT_NICE, &rl) < 0)
+    g_warning("setrlimit rtprio: %s", strerror (errno));
+
+  /* In nanoseconds */
+  rl.rlim_max = rl.rlim_cur = 30*1000*1000;
+
+  if (setrlimit(RLIMIT_RTTIME, &rl) < 0)
+    g_warning("setrlimit rttime: %s", strerror (errno));
+
+  uid = getuid();
+  if (uid != 0 && geteuid() == 0) {
+    gint ret;
+    ret = setresuid(uid, uid, uid);
+    g_assert (ret >= 0);
+    g_assert (getuid() == uid);
+    g_assert (geteuid() == uid);
+  }
+
+  g_thread_init (NULL);
+
+  optcontext = g_option_context_new ("Telepathy Stream Engine");
+  g_option_context_add_group (optcontext, gst_init_get_option_group ());
+  g_option_context_add_group (optcontext, gtk_get_option_group (TRUE));
+
+  if (g_option_context_parse (optcontext, &argc, &argv, &error) == FALSE) {
+    g_print ("%s\nRun '%s --help' to see a full list of available command line options.\n",
+        error->message, argv[0]);
+    return 1;
+  }
+  g_option_context_free (optcontext);
+
+  g_set_application_name("Maemo Telepathy Stream Engine");
+  g_setenv("PULSE_PROP_media.role", "phone", TRUE);
 
   tp_debug_divert_messages (g_getenv ("STREAM_ENGINE_LOGFILE"));
   tp_debug_set_flags (g_getenv ("STREAM_ENGINE_DEBUG"));
@@ -261,36 +221,10 @@ int main(int argc, char **argv)
   if (g_getenv ("STREAM_ENGINE_TIMING") != NULL)
     g_log_set_default_handler (tp_debug_timestamped_log_handler, NULL);
 
-  signal (SIGBUS, got_sigbus);
-
-#ifdef GST_DEBUG_BIN_TO_DOT_FILE_WITH_TS
   signal (SIGUSR1, got_sigusr1);
-#endif
 
-#ifdef USE_REALTIME
-  {
-    int rt_mode;
-    char *rt_env;
 
-    /* 3.11.2006: This has to be called before gst_init() in order to make
-     * thread pool inherit the scheduling policy. However, this breaks gthreads,
-     * so disabled for now...  -jl */
-    /* Here we don't yet have any media threads running, so the to-be-created
-     * threads will inherit the scheduling parameters, as glib doesn't know
-     * anything about that... */
-    rt_env = getenv("STREAM_ENGINE_REALTIME");
-    if (rt_env != NULL) {
-      if ((rt_mode = atoi(rt_env))) {
-        g_debug("realtime scheduling enabled");
-        set_realtime(argv[0], rt_mode);
-      } else {
-        g_debug("realtime scheduling disabled");
-      }
-    } else {
-      g_debug("not using realtime scheduling, enable through STREAM_ENGINE_REALTIME env");
-    }
-  }
-#endif /* USE_REALTIME */
+
 
   {
     GLogLevelFlags fatal_mask;
@@ -302,9 +236,9 @@ int main(int argc, char **argv)
 
   g_set_prgname("telepathy-stream-engine");
 
-  mainloop = g_main_loop_new (NULL, FALSE);
+  stream_engine_cli_init ();
 
-  dbus_g_error_domain_register (TP_ERRORS, "org.freedesktop.Telepathy.Error", TP_TYPE_ERROR);
+  mainloop = g_main_loop_new (NULL, FALSE);
 
   stream_engine = tp_stream_engine_get ();
 
@@ -327,10 +261,6 @@ int main(int argc, char **argv)
       signal (SIGALRM, watchdog_bite);
     }
 
-#ifdef MAEMO_OSSO_SUPPORT
-  g_debug ("maemo support enabled");
-#endif
-
   g_debug("started");
   g_main_loop_run (mainloop);
   g_debug("finished");
